import java.awt.*;
import java.awt.event.*;

import java.io.*;

import javax.swing.*;

import com.mindfusion.diagramming.*;

public class DirTreeFrame extends JFrame 
{
	public DirTreeFrame()
	{
		initDiagram();
		initFrame();
	}

	private void initDiagram()
	{
		diagram = new Diagram();

		diagram.getNodeEffects().add(new GlassEffect());
        diagram.getShapeNodeStyle().setFontFamily("Times New Roman");
        diagram.getShapeNodeStyle().setFontSize(10);

        diagram.setRecursiveExpand(true);
        diagram.setLinkCascadeOrientation(
        	com.mindfusion.diagramming.jlayout.Orientation.Vertical);
        diagram.setLinkHeadShape(ArrowHeads.BowArrow);
        diagram.setLinkHeadShapeSize(9);
        diagram.setMeasureUnit(GraphicsUnit.Point);
        diagram.setAdjustmentHandlesSize(6);
        diagram.setLinkShape(LinkShape.Cascading);
        diagram.setAutoResize(15);
		diagram.setLinkSegments((short)2);
		diagram.setShadowOffsetX(2);
		diagram.setShadowOffsetY(2.5f);
		
        // set folder-like polygonal shape as default for boxes
		ElementTemplate[] folderShape = new ElementTemplate[]
		{
			 new LineTemplate(0, 0, 40, 0),
			 new LineTemplate(40, 0, 55, 20),
			 new LineTemplate(55, 20, 100, 20),
			 new LineTemplate(100, 20, 100, 100),
			 new LineTemplate(100, 100, 0, 100),
			 new LineTemplate(0, 100, 0, 0)
		};
		com.mindfusion.diagramming.Shape shapeDef = new com.mindfusion.diagramming.Shape(
			folderShape, java.awt.geom.GeneralPath.WIND_EVEN_ODD, "folderShape");
		diagram.setDefaultShape(shapeDef);
		diagram.getShapeNodeStyle().setBrush(new GradientBrush(
				new Color((int)228, (int)255, (int)255),
				new Color((int)128 ,(int)192 ,(int)228), 20));
		diagram.setBackBrush(new SolidBrush(new Color((int)242, (int)242, (int)224)));

		// allow hierarchical expand/collapse
		diagram.setNodesExpandable(true);

		// read some directories
		buildTree();

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeClicked(NodeEvent e)
        	{
				if(e.getMouseButton() != MouseEvent.BUTTON3 )
					return;

    			if(e.getNode() instanceof ShapeNode)
    			{
    				Point mousePos = getMousePosition();
    				dir = (File)e.getNode().getTag();
    				cmFolder.show(DirTreeFrame.this, mousePos.x, mousePos.y);
    			}
        	}

        	public void treeCollapsed(NodeEvent e)
        	{
        		onTreeCollapsed(e);
        	}

        	public void treeExpanded(NodeEvent e)
        	{
        		onTreeExpanded(e);
        	}
        });
	}

    int height = 0;

    private void buildTree()
	{
		// create a node that will be a root in our hierarchy
		ShapeNode root = diagram.getFactory().createShapeNode(18,18,104,44);
		root.setText("root");
		height = 60;

		// read directories of all drives
		File[] fileName = File.listRoots();
		String[] drives = new String[fileName.length];
		for (int i = 0; i <= fileName.length - 1; i++)
			drives[i] = fileName[i].getAbsoluteFile().toString();

		int skip = 0;
		if (drives.length >= 2)
		{
			if(drives[0].compareTo("A:\\") == 0) skip++;
			if(drives[1].compareTo("B:\\") == 0) skip++;
		}

		File[] di = new File[drives.length - skip];
		for (int i = 0; i<di.length; i++)
			di[i] = new File(drives[i + skip]);

		buildSubDirs(root, di, 1);

		performLayout();

		// after the whole tree is built adjust the document extents
		// to be as big as needed
		diagram.resizeToFitItems(40);
	}

	private void performLayout()
	{
		TreeLayout tl = new TreeLayout();

		tl.setRoot(null);
		tl.setType(TreeLayoutType.Cascading);
		tl.setDirection(
			com.mindfusion.diagramming.jlayout.TreeLayoutDirection.LeftToRight);
		tl.setLevelDistance(-18);
		tl.setNodeDistance(15);
		tl.setKeepRootPosition(true);
		tl.setReversedLinks(false);
		tl.setLinkStyle(TreeLayoutLinkType.Cascading2);

		tl.arrange(diagram);
	}

	private void buildSubDirs(ShapeNode root, File[] dirs,  int level)
	{
		for (File dir : dirs )
		{
			ShapeNode folder = diagram.getFactory().createShapeNode(
				(18 + level * 60)*3, 3 * height, 110, 45);
			if (level == 1)
				folder.setText(dir.getAbsoluteFile().toString());
			else
				folder.setText(dir.getName().toString());

			folder.setTag(dir);	// associate dir-info with node

			// link the subfolder with its parent
			DiagramLink arrow = diagram.getFactory().createDiagramLink(root, folder);
			arrow.setSegmentCount(2);
			arrow.setShape(LinkShape.Cascading);

			// vertical offset for next folders
			height += 42;
			if (level > 2)
				continue; //avoid deep recursion

			try
			{
				// build subtrees recursively
				File[] subDirs = dir.listFiles(new FileFilter()
				{
					public boolean accept(File pathname)
					{
						return pathname.isDirectory();
					}
				});

				if (subDirs != null)
					buildSubDirs(folder, subDirs, level + 1);
			}
			catch(Exception exp)
			{
				exp.getMessage(); //skip warning
			}
		}
	}

	private void onDelete()
	{
		// our lawyers advised us not to delete anything
		JOptionPane.showConfirmDialog(dialogFrame,
			"Available in the registered version only :)", "Delete" , JOptionPane.CLOSED_OPTION);
	}

	/**
	 * Handles the TreeCollapsed event.
	 */
	private void onTreeCollapsed(NodeEvent e)
	{
		performLayout();
	}

	/**
	 * Handles the TreeExpanded event.
	 */
	private void onTreeExpanded(NodeEvent e)
	{
		performLayout();
	}

	ShapeNode currBox = null;

	private void onCreateSub()
	{
		final EnterNameForm form = new EnterNameForm();
		form.setVisible(true);
		form.setFocusable(true);
		form.ok.addActionListener(new ActionListener(){
		
			public void actionPerformed(ActionEvent e){

				if(dir == null)
				{
					form.setVisible(false);
					return;
				}

				if (form.getTextForm() != null)
				{
					File newFolder = new File(dir.getAbsolutePath() + form.getTextForm());
					
					if (newFolder.exists())
						return;
					
					newFolder.mkdir();

					// rebuild the tree after directory is created
					diagram.clearAll();
					buildTree();
				}
				form.dispose();
			}
		});
	}
	
	private void initFrame()
	{
		dialogFrame = new JFrame();
        menuItem3 = new JMenuItem();
        miCreateSub = new JMenuItem();
    	miDelete = new JMenuItem();

        cmFolder = new JPopupMenu();
        cmFolder.add(miCreateSub);
        cmFolder.add(miDelete);
        cmFolder.add(menuItem3);

        //
        //menuItem3
        //
        menuItem3.setName("menuItem3");
        menuItem3.setText("Show files");

        //
        //miDelete
        //
        miDelete.setName("miDelete");
        miDelete.setText("Delete");
        miDelete.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e){
        		onDelete();
        	}
        });

        //
        //miCreateCub
        //
        miCreateSub.setName("miCreateSub");
        miCreateSub.setText("Create subfolder");
        miCreateSub.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e){
        		onCreateSub();
        	}
        });
        
		diagramView = new DiagramView(diagram);
		diagramView.setBehavior(Behavior.Modify);
        scrPane = new JScrollPane(diagramView);
        scrPane.setAutoscrolls(true);

        Container cp = this.getContentPane();
	    cp.add(scrPane, BorderLayout.CENTER);

        setTitle("MindFusion.Diagramming Sample: Directory Tree");
        setSize(new Dimension(640, 750));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Add window listener.
        this.addWindowListener
        (
            new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                	System.exit(0);
                }
            }
        );
	}

    private JFrame dialogFrame;
    private Diagram diagram;
    private DiagramView diagramView;
    private JScrollPane scrPane;
    private JMenuItem menuItem3;
    private JMenuItem miCreateSub;
    private JMenuItem miDelete;
    private File dir; //  get the directory attached to this

    JPopupMenu cmFolder;
}