import java.awt.Color;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.DefaultCellEditor;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;


public class PropertyGrid extends JTable
{
	public PropertyGrid()
	{
		setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
		setSelectedObject(null);
	}
	
	/**
	 * Gets the object for which the grid displays properties.
	 * @return The object for which the grid displays properties. 
	 * If there is no currently selected object the return is null.
	 */
	public Object getSelectedObject()
	{
		return obj;
	}
	
	/**
	 * Sets the object for which the grid displays properties.
	 * @param obj The object for which the grid displays properties.  
	 */
	public void setSelectedObject(Object obj)
	{
		this.obj = obj;
		try 
		{
			setModel(new PropertyGridModel(obj));
		}
		catch (IntrospectionException e)
		{
			System.err.println("WARNING: can't introspect: " + obj);
		}
		
		update();
	}
	
	private void update()
	{
		// Tweak the appearance of the table by manipulating its column model
	    TableColumnModel colmodel = getColumnModel();

	    // Set column widths
	    colmodel.getColumn(0).setPreferredWidth(50);

	    colmodel.getColumn(1).setCellEditor(new DefaultCellEditor(new JTextField()));
	    setEditingColumn(0);
	    
	    // Right justify the text in the first column
	    TableColumn namecol = colmodel.getColumn(0);
	    DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
	    renderer.setHorizontalAlignment(SwingConstants.LEFT);
	    namecol.setCellRenderer(renderer);
	}
    
    static class PropertyGridModel extends AbstractTableModel
    {
    	private static final long serialVersionUID = 1L;

		public PropertyGridModel(Object object)
    		throws java.beans.IntrospectionException 
    	{
    		this.object = object;

    		array = new ArrayList<PropertyDescriptor>();
    		
    		initConverters();
    		
    		// If the object is null then only columns' names will be shown
    		if (object == null)
    			return;
    		
    		// Use the introspector class to get "bean info" about the class.
    		BeanInfo beaninfo = Introspector.getBeanInfo(object.getClass());
			
    		// Get the property descriptors from that BeanInfo class
			for (PropertyDescriptor pd : beaninfo.getPropertyDescriptors())
			{
				if (pd.getPropertyType() == null)
					continue;
				
				//for (Class c : assignableTypes)
				if (converters.containsKey(pd.getPropertyType()))
				{
					array.add(pd);
				}
			}
    	}
		
		private void initConverters()
		{
			converters = new HashMap<Class, Converter>();
			
			converters.put(String.class, null);
			converters.put(double.class, null);
			converters.put(int.class, null);
			converters.put(float.class, null);
			converters.put(boolean.class, null);
			converters.put(byte.class, null);
			converters.put(short.class, null);
			converters.put(long.class, null);
			converters.put(char.class, null);
			converters.put(Color.class, new ColorConverter());
		}

    	// These are the names of the columns represented by this TableModel
        static final String[] columnNames = new String[] { "Name", "Value" };

        // These are the types of the columns represented by this TableModel
        static final Class[] columnTypes = new Class[] { String.class, String.class };
    	
		public String getColumnName(int column) {
            return columnNames[column];
        }

		public Class getColumnClass(int column) {
        	return columnTypes[column];
        }

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return array != null ? array.size() : 0;
		}

		public Object getValueAt(int row, int column)
		{
			PropertyDescriptor prop = array.get(row);
	        switch (column) {
	        case 0:
	          return prop.getName();
	        case 1:
	          try 
	          		{
	        	  		// get the value of the property
	        	  		Object obj = prop.getReadMethod().invoke(object);
	        	  		Converter converter = converters.get(prop.getPropertyType());
	        	  		if (converter != null)
	        	  			return converter.toString(obj);

						return obj;
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
					} catch (InvocationTargetException e) {
						e.printStackTrace();
					}
	          default:
	        	  return null;
	        }
		}
		
		public boolean isCellEditable(int row, int col)
		{
            return col == 1;
        }
		
		public void setValueAt(Object aValue, int rowIndex, int columnIndex)
		{
			if (rowIndex < 0 || rowIndex >= getRowCount())
				return;
			
			// this is our column with properties' values
			if (columnIndex != 1)
				return;
			
			if (aValue == null)
				return;
			
			PropertyDescriptor prop = array.get(rowIndex);
	        
			if (prop.getWriteMethod() != null)
			{
				try
				{
					// get the value of the property
        	  		//Object obj = prop.getReadMethod().invoke(object);
        	  		Converter converter = converters.get(prop.getPropertyType());
        	  		if (converter != null)
        	  		{
        	  			prop.getWriteMethod().invoke(object, 
        	  					converter.fromString(aValue.toString()));
        	  			return;
        	  		}
        	  		
					if (prop.getPropertyType().isAssignableFrom(String.class))
						prop.getWriteMethod().invoke(object, aValue);
					else if (prop.getPropertyType().isAssignableFrom(int.class))
						prop.getWriteMethod().invoke(object, Integer.parseInt(aValue.toString()));
					else if (prop.getPropertyType().isAssignableFrom(boolean.class))
						prop.getWriteMethod().invoke(object, Boolean.parseBoolean(aValue.toString()));
					else if (prop.getPropertyType().isAssignableFrom(float.class))
						prop.getWriteMethod().invoke(object, Float.parseFloat(aValue.toString()));
					else if (prop.getPropertyType().isAssignableFrom(double.class))
						prop.getWriteMethod().invoke(object, Double.parseDouble(aValue.toString()));
					else if (prop.getPropertyType().isAssignableFrom(byte.class))
						prop.getWriteMethod().invoke(object, Byte.parseByte(aValue.toString()));
					else if (prop.getPropertyType().isAssignableFrom(short.class))
						prop.getWriteMethod().invoke(object, Short.parseShort(aValue.toString())); 
					else if (prop.getPropertyType().isAssignableFrom(long.class))
						prop.getWriteMethod().invoke(object, Long.parseLong(aValue.toString())); 
					else if (prop.getPropertyType().isAssignableFrom(char.class))
						prop.getWriteMethod().invoke(object, aValue.toString().charAt(0));
				} 
				catch (IllegalAccessException e)
				{
					return;
				}
				catch (IllegalArgumentException e)
				{
					return;
				}
				catch (InvocationTargetException e)
				{
					return;
				}
			}
		}
    	
		Object object;
		PropertyDescriptor[] properties; // The properties to display
		private ArrayList<PropertyDescriptor> array;
		
		private HashMap<Class, Converter> converters = new HashMap<Class, Converter>();
    }
    
	private static final long serialVersionUID = 1L;
	private Object obj;
}

interface Converter<E>
{
	public String toString(E e);
	
	public E fromString(String value);
}

class ColorConverter implements Converter<Color>
{
	/**
	 * Converts a color object to string representation
	 * of the color argument as an RGB hexadecimal value.
	 */
	public String toString(Color e)
	{
		if (e == null)
			return null;
		
		String a = Integer.toHexString(e.getAlpha());
		if (a.length() == 1)
			a = "0" + a;
		String r = Integer.toHexString(e.getRed());
		if (r.length() == 1)
			r = "0" + r;
		String g = Integer.toHexString(e.getGreen());
		if (g.length() == 1)
			g = "0" + g;
		String b = Integer.toHexString(e.getBlue());
		if (b.length() == 1)
			b = "0" + b;
		
		String v = "#" + a + r + g + b;
		return v.toUpperCase();
	}

	public Color fromString(String value)
	{
		value = new String(value);
		
		if (value == null)
			return null;
		
		if (value.startsWith("#"))
			value = value.substring(1);
		
		if (value.length() != 8 && value.length() == 6)
			return null;
		
		int a = 255, r = 0, g = 0, b = 0;
		if (value.length() == 8)
		{
			String v = value.substring(0, 2);
			value = value.substring(2);
			a = Integer.parseInt(v, 16);
		}
		
		String v = value.substring(0, 2);
		value = value.substring(2);
		r = Integer.parseInt(v, 16);
		
		v = value.substring(0, 2);
		value = value.substring(2);
		g = Integer.parseInt(v, 16);
		
		b = Integer.parseInt(value, 16);
		
		return new Color(r, g, b, a);
	}
}
